@@ -7,10 +7,29 @@ _KT = Any
 _VT = Any
 # _KT = Union[str, int, None]
 # _VT = Union[str, int, float, str, bool, None, list, dict, tuple, Path]
-_ErrorMessages = namedtuple(
-    "_ErrorMessages",
-    ["frozen", "fixkey", "noattrib", "noarg"]
-)(
+
+
+class ErrorMessages(namedtuple(
+    "ErrorMessages",
+    ["frozen", "fixkey", "noattrib", "noarg"])
+):
+    def _replace(self, **kwargs):
+        raise AttributeError("'ErrorMessages' has no attribute '_replace'")
+    def _make(self, values):
+        raise AttributeError("'ErrorMessages' has no attribute '_make'")
+
+
+class Options(namedtuple(
+    "Options",
+    ["frozen", "fixkey", "fixtype", "cast"])
+):
+    def _replace(self, **kwargs):
+        raise AttributeError("'Options' has no attribute '_replace'")
+    def _make(self, values):
+        raise AttributeError("'ErrorMessages' has no attribute '_make'")
+
+
+_ErrorMessages = ErrorMessages(
     frozen = "cannot assign to field of frozen instance",
     fixkey = "If fixkey, cannot add or delete keys",
     noattrib = "'rsdict' object has no attribute",
@@ -48,6 +67,8 @@ class rsdict(dict):
         >>> from rsdict import rsdict
         >>> rd = rsdict(dict(foo=1, bar="baz"))
     """
+    __initialized = False
+
     def __init__(
         self,
         items: dict,
@@ -90,35 +111,28 @@ class rsdict(dict):
         check_instance(fixtype, int, classname="bool")
         check_instance(cast, int, classname="bool")
 
-        super().__init__(items)
-
         # Store initial values
-        InitialValues = namedtuple(
-            "InitialValues",
-            ["items", "frozen", "fixkey", "fixtype", "cast"]
-        )
-        self.__initval = InitialValues(
-            items = items.copy(),
+        self.__options = Options(
             frozen = bool(frozen),
             fixkey = bool(fixkey),
             fixtype = bool(fixtype),
             cast = bool(cast),
         )
+        self.__inititems = items.copy()
+
+        self.__initialized = True
+        return super().__init__(items)
 
     @check_option("fixkey")
-    def _addkey(self, key: _KT, value: _VT) -> None:
+    def __addkey(self, key: _KT, value: _VT) -> None:
         # add initialized key
-        items = self.get_initial()
-        items[key] = value
-        self.__initval = self.__initval._replace(items = items)
+        self.__inititems[key] = value
         return super().__setitem__(key, value)
 
     @check_option("fixkey")
-    def _delkey(self, key: _KT) -> None:
+    def __delkey(self, key: _KT) -> None:
         # delete initialized key
-        items = self.get_initial()
-        del items[key]
-        self.__initval = self.__initval._replace(items = items)
+        del self.__inititems[key]
         # delete current key
         return super().__delitem__(key)
 
@@ -132,7 +146,7 @@ class rsdict(dict):
             TypeError: If fixtype and not cast and type(value)!=type(initial_value).
             ValueError: If fixtype and failed in casting.
         """
-        if key in self.keys():
+        if key in self:
             initialtype = type(self.get_initial(key))
             if type(value) is initialtype:
                 # type(value) is same as type(initial value)
@@ -152,30 +166,21 @@ class rsdict(dict):
             return super().__setitem__(key, value)
         else:
             # add a new key
-            return self._addkey(key, value)
+            return self.__addkey(key, value)
 
     @check_option("frozen")
     def __delitem__(self, key: _KT) -> None:
         """Cannot delete if fixkey or frozen."""
-        return self._delkey(key)
+        return self.__delkey(key)
 
-    def __getattribute__(self, name: str) -> Any:
-        # disable some attributes (of built-in dictionary)
-        if name in ["fromkeys"]:
-            raise AttributeError(
-                "{} '{}'".format(
-                    _ErrorMessages.noattrib,
-                    name,
-                )
-            )
-        return super().__getattribute__(name)
+    # def __getattribute__(self, name: str) -> Any:
+    #     return super().__getattribute__(name)
 
     def __setattr__(self, name: str, value: Any) -> None:
-        enable = name in (
-            dir(self) + ["_rsdict__initval"])
-
-        if enable:
-            return super().__setattr__(name, value)
+        if name in dir(self) and not name.startswith("_rsdict__"):
+            pass
+        elif not self.__initialized:
+            pass
         else:
             raise AttributeError(
                 "{} '{}'".format(
@@ -183,6 +188,7 @@ class rsdict(dict):
                     name,
                 )
             )
+        return super().__setattr__(name, value)
 
     def __sizeof__(self) -> int:
         """Return size(current values) + size(initial values)"""
@@ -223,7 +229,7 @@ class rsdict(dict):
             else:
                 newkeys = (other.keys() | self.keys()) - self.keys()
                 for key in newkeys:
-                    self._addkey(key, other[key])
+                    self.__addkey(key, other[key])
                 return super().__ior__(other)
 
         # def __ror__(self, other):
@@ -272,6 +278,7 @@ class rsdict(dict):
             `reset=False, frozen=True` option,
             current values are copied as initial values and frozen.
         """
+        check_instance(reset, int, classname="bool")
         if frozen is None:
             frozen = bool(self._get_option("frozen"))
         if fixkey is None:
@@ -285,7 +292,7 @@ class rsdict(dict):
             # initialize with current values
             items = self.to_dict().copy()
         else:
-            # initialize with initval values
+            # initialize with initial values
             items = self.get_initial()
 
         # create new instance
@@ -302,7 +309,7 @@ class rsdict(dict):
             pass
         else:
             # copy current values
-            for key in self.keys():
+            for key in self:
                 rd[key] = self[key]
         return rd
 
@@ -315,9 +322,7 @@ class rsdict(dict):
     @check_option("fixkey")
     def clear(self) -> None:
         # clear initialized key
-        items = self.get_initial()
-        items.clear()
-        self.__initval = self.__initval._replace(items = items)
+        self.__inititems.clear()
         # clear current key
         return super().clear()
 
@@ -339,6 +344,9 @@ class rsdict(dict):
     def popitem(self) -> tuple:
         return super().popitem()
 
+    def fromkeys(self, keys, value):
+        raise AttributeError(_ErrorMessages.noattrib + "'fromkeys'")
+
     # TODO: (optional) check frozen deco
     def reset(self, key: _KT = None) -> None:
         """Reset values to initial values.
@@ -369,25 +377,12 @@ class rsdict(dict):
             Any (else): Initial value.
         """
         if key is None:
-            return self.__initval.items
+            return self.__inititems
         else:
-            return self.__initval.items[key]
+            return self.__inititems[key]
 
     def _get_option(self, name: str) -> bool:
-        if name in ["items"]:
-            # NOTE: items is in initval but not 'option'
-            raise AttributeError(
-                "'{}' is not option".format(name)
-            )
-        elif name not in self.__initval._fields:
-            raise AttributeError(
-                "{} '{}'".format(
-                    _ErrorMessages.noarg,
-                    name,
-                )
-            )
-        else:
-            return self.__initval.__getattribute__(name)
+        return self.__options.__getattribute__(name)
 
     def is_changed(self) -> bool:
         """Return whether the values are changed.
